unit KLSFile;

interface

uses
	Classes, SysUtils, ID3v2, TntClasses;

type

  kls_header = packed record
    FormatTag:  Word;	//1Œ
    Channels:	Word;	//`l
    SamplesPerSec:  Longword;	//TvO[g
    AvgBytesPerSec: Longword;	//f[^x
    BlockAlign:	Word; //ubNTCY
    BitsPerSample:	Word;	//Tṽrbg
    nazo1:	Word;	//0Œ?
    KENWOOD: array[0..7] of Char; //"KENWOOD"
    nazo2:  LongWord; //18 00 00 00ŒH
    nazo3:  LongWord; //00 00 20 00ŒH
    Samples:  LongWord; //t@CŜł̃Tv
    nazo4:  Longword; //?????
    nazo5:  LongWord; //?????
    OriginalDataSize:  Longword; //WAVt@C̃f[^̃TCY
  end;

  kls_header_chunk = packed record
    KLSid: array[0..3] of Char;
    nazo: Word;
    chunk_size: Longword;
  end;

	{ Class TKLS }
  TKLS = class(TObject)
  private
    { Private declarations }
    FFileSize: Int64;
    FValid: Boolean;

    FAudioFormat: Word;
    FChannels: Word;
    FBits: Word;
    FSamples: Longword;
    FSampleRate: Longword;

    FBitrate: Double;
    FDuration: Double;

    FID3v2: TID3v2;

    function FGetRatio: Double;
    procedure FResetData;

  public
    { Public declarations }
    constructor Create;                                     { Create object }
    destructor Destroy; override;                          { Destroy object }

    function ReadFromFile(const FileName: WideString): Boolean;   { Load header }

    property FileSize: Int64 read FFileSize;
    property Valid: Boolean	read FValid;

    property AudioFormat: Word read FAudioFormat;
    property Channels: Word read FChannels;
    property Bits: Word read FBits;
    property SampleRate: Longword read FSampleRate;
    property Samples: Longword read FSamples;           { Number of samples }

    property Bitrate: Double read FBitrate;
    property Duration: Double read FDuration;
    property Ratio: Double read FGetRatio;          { Compression ratio (%) }

    property ID3v2: TID3v2 read FID3v2;                    { ID3v2 tag data }
	end;

implementation


{ ********************** Private functions & procedures ********************* }

procedure TKLS.FResetData;
begin
	{ Reset all data }
  FFileSize := 0;
  FValid := False;

  FAudioFormat := 0;
  FChannels := 0;
  FBits := 0;
  FSampleRate := 0;
  FSamples := 0;

  FBitrate := 0;
  FDuration := 0;

  FID3v2.ResetData;
end;


{ ********************** Public functions & procedures ********************** }

constructor TKLS.Create;
begin
  { Create object }
  inherited;
  FID3v2   := TID3v2.Create;
  FResetData;
end;

(* -------------------------------------------------------------------------- *)

destructor TKLS.Destroy;
begin
  FID3v2.Free;
  inherited;
end;

(* -------------------------------------------------------------------------- *)

function TKLS.ReadFromFile(const FileName: WideString): Boolean;
var
  f: TTntFileStream;
  klsheader: kls_header;
  klsheaderchunk: kls_header_chunk;
  TagSize: Int64;
begin
  Result := False;
  FResetData;
  // load tags first
  FID3v2.ReadFromFile(FileName);
  // calulate total tag size
  TagSize := 0;
  if FID3v2.Exists then inc(TagSize, FID3v2.Size);
  // begin reading data from file
  f:=nil;

  try
    f := TTntFileStream.create(FileName, fmOpenRead or fmShareDenyWrite);
    //ID3^Oǂݔ΂B
    if FID3v2.Exists then
    begin
      f.Seek(FID3v2.Size, soFromBeginning);
    end;

    if (f.Read(klsheaderchunk, SizeOf(klsheaderchunk)) = SizeOf(klsheaderchunk)) and (StrLComp(klsheaderchunk.KLSID,'KLS',3) = 0) then
    begin
      // start looking for chunks
			FillChar(klsheader, SizeOf(klsheader),0);
      f.Read(klsheader, SizeOf(klsheader));

      FFileSize := f.Size;
      FValid := TRUE;

      FAudioFormat := klsheader.FormatTag;
      FChannels := klsheader.Channels;

      FBits := klsheader.BitsPerSample;
      FSampleRate := klsheader.SamplesPerSec;
      FSamples := klsheader.Samples;

      FBitrate := FFileSize * 8 / (FSamples / FSampleRate);
      FDuration := klsheader.Samples / klsheader.SamplesPerSec;

      Result := True;
    end;

  finally
    f.free;
  end;
end;

(* -------------------------------------------------------------------------- *)
(*k擾*)
function TKLS.FGetRatio: Double;
begin
  { Get compression ratio }
  if FValid then
    Result := FFileSize / (FSamples * (FChannels * FBits / 8) + 44) * 100
  else
    Result := 0;
end;

(* -------------------------------------------------------------------------- *)

end.
